//------------------------------------------------------------------------------
// Keil Software, Inc.
// 
// Project: 	I2C EXAMPLE PROGRAM FOR T8xC251G1D MCU
//
// Filename: 	T8xC251G1D_I2C_Example.c
// Version: 	1.0.0
// Description:	This example will test the I2C perpherial of the T8xC251G1D.
//				This will be done by writing and reading from
//				a serial A/D & D/A (P8591).
//				
//				The example work with communicate at: 9600, 8, N, 1.
//
// Copyright 2000 - Keil Software, Inc.
// All rights reserved.
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Header files
//------------------------------------------------------------------------------
#include "reg251g1.h" 						// Header file for the T8xC251G1D MCU
#include <STDIO.H>							// Standard I/O header file

#define TRUE 	1
#define FALSE 	0

//------------------------------------------------------------------------------
// I2C Peripheral Function Prototypes
//------------------------------------------------------------------------------
											// Write a byte to memory over I2C
void write_byte (unsigned char data_out, unsigned int address);
											// Read a byte from memory over I2C
unsigned char read_byte (unsigned int address);


//------------------------------------------------------------------------------
// I2C Function Prototypes
//------------------------------------------------------------------------------
void i2c_initialize (void);					// Initializes the I2C Port

//------------------------------------------------------------------------------
// Support Function Prototypes
//------------------------------------------------------------------------------
void initialize_serial (void);				// Initializes the UART (RS-232)


//------------------------------------------------------------------------------
// Local Global Variables for I2C Functions and the ISR
//------------------------------------------------------------------------------
bit i2c_busy, data_ok, send_rep_start;
unsigned char i2c_slave_address;
unsigned char i2c_data[5];
unsigned char i2c_byte_index;
unsigned char i2c_byte_count;

//------------------------------------------------------------------------------
// Macros
//------------------------------------------------------------------------------
											// Get high byte macro
#define high_byte(x)		((x & 0xFF00) >> 8)


//------------------------------------------------------------------------------
// MAIN FUNCTION 
//------------------------------------------------------------------------------
void main (void)
{
	unsigned int eeprom_address;
	unsigned char eeprom_data;

	initialize_serial();					// Initialize serial port (UART - 9600 bps)
	i2c_initialize();						// Initialize I2C port (100 kHz)

	EA = 1;									// Enable the interrupts for the system

	printf("\n\rKeil Software, Inc.\n\r");	// Display starup message
	printf("T8xC251G1D MCU IC Example Test Program\n\r");
	printf("Version 1.0.0\n\r");
	printf("Copyright 2002 - Keil Software, Inc.\n\r");
	printf("All rights reserved.\n\n\r");

	printf("Writing data to EEPROM....");

	for (eeprom_address = 0; eeprom_address < 256; eeprom_address++) 
	{
		write_byte(~eeprom_address, eeprom_address);
	}
	printf("Done!\r\n\n");

	while (TRUE)							// Begin infinite loop
	{
		for (eeprom_address = 0; eeprom_address < 256; eeprom_address++)
		{
											// Read data from eeprom and display it
			eeprom_data = read_byte(eeprom_address);
			printf("Address: 0x%X  -  Data Read: 0x%bX\n\r", eeprom_address, eeprom_data);
		}
	}

}

//------------------------------------------------------------------------------
// I2C Peripheral Function Prototypes
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Procedure:	write_byte
// Inputs:		data out, address
// Outputs:		none
// Purpose:		Writes a byte to the EEPROM given the address 
//------------------------------------------------------------------------------
void write_byte (unsigned char data_out, unsigned int address)
{
	data_ok = FALSE;
	i2c_slave_address = 0x50;						// The I2C slave address of EEPROM (0x50)
	i2c_data[0] = high_byte(address);				// High byte of the memory address
	i2c_data[1] = (unsigned char)address;			// Low byte of the memory address
	i2c_data[2] = data_out;							// Data byte to write to memory
	i2c_byte_count = 3;								// # of bytes to write over I2C
	send_rep_start = FALSE;
	data_ok = TRUE;
	i2c_busy = TRUE;
	SSSTA = TRUE;									// Perform I2C start, send data over bus
	while (data_ok & i2c_busy);						// Wait until packet is sent, sucessfull or not
}

//------------------------------------------------------------------------------
// Procedure:	read_byte
// Inputs:		address
// Outputs:		data
// Purpose:		Reads a byte from the EEPROM given the address 
//------------------------------------------------------------------------------
unsigned char read_byte (unsigned int address)
{
   	unsigned char data_in;

	data_ok = FALSE;
	i2c_slave_address = 0x50;						// The I2C slave address of EEPROM (0x50)
	i2c_data[0] = high_byte(address);				// High byte of the memory address
	i2c_data[1] = (unsigned char)address;			// Low byte of the memory address
	i2c_byte_count = 2;								// # of bytes to write over I2C
	send_rep_start = TRUE;							// Send repeated start command (we wrote 2 bytes now we want to read)
	data_ok = TRUE;
	i2c_busy = TRUE;

	SSSTA = TRUE;									// Perform I2C start, send data over bus
	while (data_ok & i2c_busy);						// Wait until packet is sent, sucessfull or not

	data_in = i2c_data[0];							// Return the data
  	return data_in;                 
}


//------------------------------------------------------------------------------
// I2C Function Prototypes
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
// 	Routine:	i2c_initialize
//	Inputs:		none
//	Outputs:	none
//	Purpose:	Initializes the I2C port for this MCU
//------------------------------------------------------------------------------
void i2c_initialize (void)
{
	// Initialize I2C Interface (Bit Rate = 93.75 kHz)
	SSBRS = 0;								// Use internal bit rate source
	SSCR2 = 0;								// Set the 93.75 kHz mode SSCR2..0 = 0x00
	SSCR1 = 0;								// See Table 8.1 in data sheet for other bit rates
	SSCR0 = 0;
	SSPE = 1;								// Enable the I2C Port
	SSSTA = 0;								// Clear start
	SSSTO = 0;								// Clear stop
	SSI = 0;								// Clear interrupt flag

	SSIE = 1;								// Enable the SSLC Interrupt
}

//------------------------------------------------------------------------------
// 	Routine:	i2c_isr
//	Inputs:		none
//	Outputs:	none
//	Purpose:	ISR for I2C Status codes
//------------------------------------------------------------------------------
void i2c_isr (void) interrupt 13
{
	switch (SSCS)
	{
		case 0x08:													// START condition
			SSDAT = (i2c_slave_address << 1);						// Send the address + W
			SSSTA = FALSE;		
			break;

		case 0x10:													// REPEATED START condition
			SSDAT = ((i2c_slave_address << 1) | 0x01);				// Send the address + R
			SSSTA = FALSE;		
			break;

		case 0x38:													// ARB LOST condition
			SSSTO = TRUE;											// Reset the bus	
			data_ok = FALSE;
			i2c_busy = FALSE;
			break;

		// MASTER TRANSMITTER MODE													
		case 0x18:													// ADDRESS OK condition
			i2c_byte_index = 0;
			if (i2c_byte_count > 0) SSDAT = i2c_data[i2c_byte_index++];	
			data_ok = TRUE;
			break;

		case 0x20:													// ADDRESS NACK condition
			SSSTO = TRUE;											// Reset the bus	
			data_ok = FALSE;
			i2c_busy = FALSE;
			break;

		case 0x28:													// DATA OK condition
			if (i2c_byte_count > i2c_byte_index) SSDAT = i2c_data[i2c_byte_index++];
			else
			{
				if (send_rep_start) SSSTA = TRUE;					// Send repeated start
				else
				{
					SSSTO = TRUE;									// Send STOP condition
					i2c_busy = FALSE;
				}
			}	
			data_ok = TRUE;
			break;

		case 0x30:													// DATA NACK condition
			SSSTO = TRUE;											// Reset the bus	
			data_ok = FALSE;
			i2c_busy = FALSE;
			break;


		// MASTER RECEIVER MODE													
		case 0x40:													// ADDRESS OK condition read
			i2c_byte_index = 0;
			SSAA = 0;
			data_ok = TRUE;
			break;

		case 0x48:													// ADDRESS NACK condition read
			SSSTO = TRUE;											// Reset the bus	
			data_ok = FALSE;
			i2c_busy = FALSE;
			break;

		case 0x50:													// DATA with ACK, not possible in this case
			SSSTO = TRUE;											// Reset the bus	
			data_ok = FALSE;
			i2c_busy = FALSE;
			break;

		case 0x58:													// DATA NACK condition
			i2c_data[i2c_byte_index++] = SSDAT;
			SSSTO = TRUE;											// Send STOP condition
			i2c_busy = FALSE;
			data_ok = TRUE;
			break;

		// ERROR CONDITIONS
		case 0x00:													// BUS ERROR condition
			i2c_busy = FALSE;
			data_ok = FALSE;
			SSSTO = TRUE;											// Reset the bus	
			break;

		default:
			break;
	}
	
	SSI = 0;														// Clear the I2C interrupt
}

//------------------------------------------------------------------------------
// SUPPORT FUNCTIONS
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Procedure:	initialize_serial
// Inputs:		none
// Outputs:		none
// Description:	Initializes serail port for MCU
//				FOsc = 24 MHz for this example
//------------------------------------------------------------------------------
void initialize_serial (void)
{

	// Initialize the serial port (9600, 8, N, 1) 
											// If SPD = 1 Then Baud Rate = FOsc / (4 * (256 - BRL))
											// If SPD = 0 Then Baud Rate = FOsc / (24 * (256 - BRL))

											// For this example:
											// BRL = 256 - (24 MHz / (24 * 9600 bps)) = 152
	SRC = 1;								// Set the Baud Rate source to be the Baud Rate Generator
	SPD = 0;								// Set the clock divider to divide by six 
	BRR = 1;								// Start the BRG
	TBCK = 1;                               // Use the BRG for Transmit
	RBCK = 1;                               // Use the BRG for Receive
	BRL = 152;								// Set the BR reload register
	SCON = 0x50;							// 0101,0000 (Mode 1 and RxD enable)
	TI = 1;									// Set the txd interrupt for first character
}
